@@ -48,6 +48,7 @@ void CoProcess::launch(const char **argv, int timeout, int infd, int outfd)
     setbuf(d_fp,0); // no buffering please, confuses select
   }
   else if(!d_pid) { // child
+    signal(SIGCHLD, SIG_DFL); // silence a warning from perl
     close(d_fd1[1]);
     close(d_fd2[0]);
 
@@ -99,7 +100,7 @@ void CoProcess::checkStatus()
       string reason="CoProcess died on receiving signal "+itoa(sig);
 #ifdef WCOREDUMP
       if(WCOREDUMP(status)) 
-	reason+=". Dumped core";
+        reason+=". Dumped core";
 #endif
       
       throw AhuException(reason);
@@ -128,9 +129,8 @@ void CoProcess::send(const string &snd)
 
 void CoProcess::receive(string &receive)
 {
-  char line[1024];
-  memset(line,0,1024);
-  
+  receive.clear();
+    
   if(d_timeout) {
     struct timeval tv={tv_sec: d_timeout, tv_usec: 0,};
     fd_set rds;
@@ -143,15 +143,12 @@ void CoProcess::receive(string &receive)
       throw AhuException("Timeout waiting for data from coprocess");
   }
 
-  if(!fgets(line,1023,d_fp))
+  if(!stringfgets(d_fp, receive))
     throw AhuException("Child closed pipe");
   
-  char *p;
-  if((p=strrchr(line,'\n')))
-    *p=0;
-
-  receive=line;
+  trim_right(receive);
 }
+
 void CoProcess::sendReceive(const string &snd, string &rcv)
 {
   checkStatus();
