@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002-2009  PowerDNS.COM BV
+    Copyright (C) 2002-2011  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -42,7 +42,7 @@ void CommunicatorClass::retrievalLoopThread(void)
     {
       Lock l(&d_lock);
       if(d_suckdomains.empty()) 
-	continue;
+        continue;
 	
       sr=d_suckdomains.front();
       d_suckdomains.pop_front();
@@ -75,7 +75,7 @@ void CommunicatorClass::mainloop(void)
     L<<Logger::Error<<"Master/slave communicator launching"<<endl;
     PacketHandler P;
     d_tickinterval=::arg().asNum("slave-cycle-interval");
-    makeNotifySocket();
+    makeNotifySockets();
 
     int rc;
     time_t next, tick;
