@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002  PowerDNS.COM BV
+    Copyright (C) 2002 - 2007 PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -56,18 +56,30 @@ QType::QType()
       insert("AAAA",28);
       insert("LOC",29);
       insert("SRV",33);
+      insert("CERT", 37);
       insert("A6",38);
       insert("NAPTR",35);
+      insert("DS", 43);
+      insert("SSHFP", 44);
+      insert("RRSIG", 46);
+      insert("NSEC", 47);
+      insert("DNSKEY", 48);
+      insert("NSEC3", 50);
+      insert("NSEC3PARAM", 51);
       insert("SPF",99);
+      insert("IXFR",251);
       insert("AXFR",252);
       insert("ANY",255);
       insert("URL",256);
       insert("MBOXFW",257);
       insert("CURL",258);
+      insert("ADDR",259);
+      insert("DLV",32769);
+      insert("TLSA",65468);
     }
 }
 
-int QType::getCode() const
+uint16_t QType::getCode() const
 {
   return code;
 }
@@ -82,7 +94,7 @@ const string QType::getName() const
   return "#"+itoa(code);
 }
 
-QType &QType::operator=(int n)
+QType &QType::operator=(uint16_t n)
 {
   code=n;
   return *this;
@@ -95,7 +107,14 @@ int QType::chartocode(const char *p)
   for(pos=names.begin();pos<names.end();++pos)
     if(pos->first==p)
       return pos->second;
+  
+  if(*p=='#') {
+    return atoi(p+1);
+  }
 
+  if(boost::starts_with(p, "TYPE"))
+    return atoi(p+4);
+    
   return 0;
 }
 
@@ -117,108 +136,8 @@ QType &QType::operator=(const string &s)
 }
 
 
-QType::QType(int n)
+QType::QType(uint16_t n)
 {
   QType();
   code=n;
 }
-
-QType::QType(const char *p)
-{
-  QType();
-  code=chartocode(p);
-}
-
-string DNSResourceRecord::serialize() const
-{
-  ostringstream ostr;
-  ostr<<escape(qname)<<"|"<<qtype.getName()<<"|"<<escape(content)<<"|"<<ttl<<"|"<<priority<<"|"<<domain_id
-     <<"|"<<last_modified;
-  return ostr.str();
-}
-
-string DNSResourceRecord::escape(const string &name) const
-{
-  string a;
-
-  for(string::const_iterator i=name.begin();i!=name.end();++i)
-    if(*i=='|' || *i=='\\'){
-      a+='\\';
-      a+=*i;
-    }
-    else
-      a+=*i;
-
-  return a;
-}
-
-int DNSResourceRecord::unSerialize(const string &source)
-{
-  // qname|qtype|content|ttl|priority|domain_id|last_modified;
-  string chunk;
-  unsigned int m=0;
-  for(int n=0;n<7;++n) {
-    chunk="";
-    for(;m<source.size();++m) {
-      if(source[m]=='\\' && m+1<source.size()) 
-	chunk.append(1,source[++m]);
-      else if(source[m]=='|') {
-	++m;
-	break;
-      }
-      else 
-	chunk.append(1,source[m]);
-    }
-    switch(n) {
-    case 0:
-      qname=chunk;
-      break;
-    case 1:
-      qtype=chunk;
-      break;
-    case 2:
-      content=chunk;
-      break;
-    case 3:
-      ttl=atoi(chunk.c_str());
-      break;
-    case 4:
-      priority=atoi(chunk.c_str());
-      break;
-    case 5:
-      domain_id=atoi(chunk.c_str());
-      break;
-    case 6:
-      last_modified=atoi(chunk.c_str());
-      break;
-    }
-  }
-  return m;
-}
-
-
-#if 0
-int main(int argc, char **argv)
-{
-  QType t;
-
-  cout << endl;
-  cout << "Assiging a '6'" << endl;
-  t=6;
-  cout << "Code is now " << t.getCode() << endl;
-  cout << "Name is now " << t.getName() << endl;
-
-  cout << endl;
-
-  cout << "Assiging a 'CNAME'" << endl;
-  t="CNAME";
-  cout << "Code is now " << t.getCode() << endl;
-  cout << "Name is now " << t.getName() << endl;
-
-  QType u;
-  u="SOA";
-  cout << u.getCode() << endl;
-
-
-}
-#endif
