@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2005 - 2008  PowerDNS.COM BV
+    Copyright (C) 2005 - 2011  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -15,6 +15,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
+#include <boost/archive/binary_iarchive.hpp>
+#include <boost/archive/binary_oarchive.hpp>
+
 #include "packetcache.hh"
 #include "utility.hh"
 
@@ -25,12 +28,12 @@
 #include <string>
 #include <map>
 #include <sys/types.h>
-
+#include <sstream>
 #include <errno.h>
 #include <iostream>
 #include <sstream>
 #include <functional>
-
+#include <boost/foreach.hpp>
 #include "dns.hh"
 #include "arguments.hh"
 #include "dnsbackend.hh"
@@ -38,6 +41,7 @@
 #include "dnspacket.hh"
 #include "logger.hh"
 #include "statbag.hh"
+#include <boost/serialization/vector.hpp>
 
 
 extern StatBag S;
@@ -46,7 +50,6 @@ vector<UeberBackend *>UeberBackend::instances;
 pthread_mutex_t UeberBackend::instances_lock=PTHREAD_MUTEX_INITIALIZER;
 
 sem_t UeberBackend::d_dynserialize;
-string UeberBackend::programname;
 string UeberBackend::s_status;
 
 // initially we are blocked
@@ -98,6 +101,80 @@ bool UeberBackend::getDomainInfo(const string &domain, DomainInfo &di)
   return false;
 }
 
+int UeberBackend::addDomainKey(const string& name, const KeyData& key)
+{
+  int ret;
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if((ret = db->addDomainKey(name, key)) >= 0)
+      return ret;
+  }
+  return -1;
+}
+bool UeberBackend::getDomainKeys(const string& name, unsigned int kind, std::vector<KeyData>& keys)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->getDomainKeys(name, kind, keys))
+      return true;
+  }
+  return false;
+}
+
+bool UeberBackend::getDomainMetadata(const string& name, const std::string& kind, std::vector<std::string>& meta)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->getDomainMetadata(name, kind, meta))
+      return true;
+  }
+  return false;
+}
+
+bool UeberBackend::setDomainMetadata(const string& name, const std::string& kind, const std::vector<std::string>& meta)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->setDomainMetadata(name, kind, meta))
+      return true;
+  }
+  return false;
+}
+
+bool UeberBackend::activateDomainKey(const string& name, unsigned int id)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->activateDomainKey(name, id))
+      return true;
+  }
+  return false;
+}
+
+bool UeberBackend::deactivateDomainKey(const string& name, unsigned int id)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->deactivateDomainKey(name, id))
+      return true;
+  }
+  return false;
+}
+
+bool UeberBackend::removeDomainKey(const string& name, unsigned int id)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->removeDomainKey(name, id))
+      return true;
+  }
+  return false;
+}
+
+
+bool UeberBackend::getTSIGKey(const string& name, string* algorithm, string* content)
+{
+  BOOST_FOREACH(DNSBackend* db, backends) {
+    if(db->getTSIGKey(name, algorithm, content))
+      return true;
+  }
+  return false;
+}
+
+
 void UeberBackend::reload()
 {
   for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
@@ -145,21 +222,19 @@ bool UeberBackend::getSOA(const string &domain, SOAData &sd, DNSPacket *p)
   d_question.zoneId=-1;
     
   if(sd.db!=(DNSBackend *)-1) {
-    int cstat=cacheHas(d_question,d_answer);
-    if(cstat==0) {
+    int cstat=cacheHas(d_question,d_answers);
+    if(cstat==0) { // negative
       return false;
     }
-    else if(cstat==1) {
-      // ehm 
-      fillSOAData(d_answer.content,sd);
-      sd.domain_id=d_answer.domain_id;
-      sd.ttl=d_answer.ttl;
+    else if(cstat==1 && !d_answers.empty()) {
+      fillSOAData(d_answers[0].content,sd);
+      sd.domain_id=d_answers[0].domain_id;
+      sd.ttl=d_answers[0].ttl;
       sd.db=0;
       return true;
     }
   }
     
-
   for(vector<DNSBackend *>::const_iterator i=backends.begin();i!=backends.end();++i)
     if((*i)->getSOA(domain, sd, p)) {
       DNSResourceRecord rr;
@@ -168,7 +243,9 @@ bool UeberBackend::getSOA(const string &domain, SOAData &sd, DNSPacket *p)
       rr.content=serializeSOAData(sd);
       rr.ttl=sd.ttl;
       rr.domain_id=sd.domain_id;
-      addOneCache(d_question,rr);
+      vector<DNSResourceRecord> rrs;
+      rrs.push_back(rr);
+      addCache(d_question, rrs);
       return true;
     }
 
@@ -190,14 +267,8 @@ void UeberBackend::setStatus(const string &st)
   s_status=st;
 }
 
-UeberBackend::UeberBackend()
-{
-  UeberBackend("default");
-}
-
 UeberBackend::UeberBackend(const string &pname)
 {
-  programname=pname;
   pthread_mutex_lock(&instances_lock);
   instances.push_back(this); // report to the static list of ourself
   pthread_mutex_unlock(&instances_lock);
@@ -205,7 +276,7 @@ UeberBackend::UeberBackend(const string &pname)
   tid=pthread_self(); 
   stale=false;
 
-  backends=BackendMakers().all();
+  backends=BackendMakers().all(pname=="key-only");
 }
 
 void UeberBackend::die()
@@ -234,7 +305,8 @@ void UeberBackend::cleanup()
 // silly Solaris fix
 #undef PC
 
-int UeberBackend::cacheHas(const Question &q, DNSResourceRecord &rr)
+// returns -1 for miss, 0 for negative match, 1 for hit
+int UeberBackend::cacheHas(const Question &q, vector<DNSResourceRecord> &rrs)
 {
   extern PacketCache PC;
   static unsigned int *qcachehit=S.getPointer("query-cache-hit");
@@ -252,7 +324,6 @@ int UeberBackend::cacheHas(const Question &q, DNSResourceRecord &rr)
   //  L<<Logger::Warning<<"looking up: '"<<q.qname+"'|N|"+q.qtype.getName()+"|"+itoa(q.zoneId)<<endl;
 
   bool ret=PC.getEntry(q.qname, q.qtype, PacketCache::QUERYCACHE, content, q.zoneId);   // think about lowercasing here
-
   if(!ret) {
     (*qcachemiss)++;
     return -1;
@@ -260,7 +331,11 @@ int UeberBackend::cacheHas(const Question &q, DNSResourceRecord &rr)
   (*qcachehit)++;
   if(content.empty()) // negatively cached
     return 0;
-  rr.unSerialize(content);
+  
+  std::istringstream istr(content);
+  boost::archive::binary_iarchive boa(istr);
+  rrs.clear();
+  boa >> rrs;
   return 1;
 }
 
@@ -270,20 +345,29 @@ void UeberBackend::addNegCache(const Question &q)
   static int negqueryttl=::arg().asNum("negquery-cache-ttl");
   if(!negqueryttl)
     return;
-  //  L<<Logger::Warning<<"negative inserting: "<<q.qname+"|N|"+q.qtype.getName()+"|"+itoa(q.zoneId)<<endl;
   PC.insert(q.qname, q.qtype, PacketCache::QUERYCACHE, "", negqueryttl, q.zoneId);
 }
 
-void UeberBackend::addOneCache(const Question &q, const DNSResourceRecord &rr)
+void UeberBackend::addCache(const Question &q, const vector<DNSResourceRecord> &rrs)
 {
   extern PacketCache PC;
   static int queryttl=::arg().asNum("query-cache-ttl");
   if(!queryttl)
     return;
+  
   //  L<<Logger::Warning<<"inserting: "<<q.qname+"|N|"+q.qtype.getName()+"|"+itoa(q.zoneId)<<endl;
-  PC.insert(q.qname, q.qtype, PacketCache::QUERYCACHE, rr.serialize(), queryttl, q.zoneId);
+  std::ostringstream ostr;
+  boost::archive::binary_oarchive boa(ostr);
+  
+  boa << rrs;
+  PC.insert(q.qname, q.qtype, PacketCache::QUERYCACHE, ostr.str(), queryttl, q.zoneId);
 }
 
+void UeberBackend::alsoNotifies(const string &domain, set<string> *ips)
+{
+  for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
+    (*i)->alsoNotifies(domain,ips);
+}
 
 UeberBackend::~UeberBackend()
 {
@@ -325,23 +409,25 @@ void UeberBackend::lookup(const QType &qtype,const string &qname, DNSPacket *pkt
     d_question.qtype=qtype;
     d_question.qname=qname;
     d_question.zoneId=zoneId;
-    int cstat=cacheHas(d_question,d_answer);
+    int cstat=cacheHas(d_question, d_answers);
     if(cstat<0) { // nothing
       d_negcached=d_cached=false;
+      d_answers.clear(); 
       (d_handle.d_hinterBackend=backends[d_handle.i++])->lookup(qtype, qname,pkt_p,zoneId);
     } 
     else if(cstat==0) {
       d_negcached=true;
       d_cached=false;
+      d_answers.clear();
     }
     else {
       d_negcached=false;
       d_cached=true;
+      d_cachehandleiter = d_answers.begin();
     }
   }
 
   d_handle.parent=this;
-
 }
 
 bool UeberBackend::get(DNSResourceRecord &rr)
@@ -351,24 +437,22 @@ bool UeberBackend::get(DNSResourceRecord &rr)
   }
 
   if(d_cached) {
-    rr=d_answer;
-    d_negcached=true; // ugly, confusing 
-    return true;
+    if(d_cachehandleiter != d_answers.end()) {
+      rr=*d_cachehandleiter++;;
+      return true;
+    }
+    return false;
   }
   if(!d_handle.get(rr)) {
     if(!d_ancount && !d_handle.qname.empty()) // don't cache axfr
       addNegCache(d_question);
 
-    if(d_ancount==1) {
-      addOneCache(d_question, lastrr);
-    }
-
+    addCache(d_question, d_answers);
+    d_answers.clear();
     return false;
   }
-
-  if(!d_ancount++) {
-    lastrr=rr;
-  }
+  d_ancount++;
+  d_answers.push_back(rr);
   return true;
 }
 
@@ -393,10 +477,6 @@ UeberBackend::handle::~handle()
   instances--;
 }
 
-
-
-
-
 bool UeberBackend::handle::get(DNSResourceRecord &r)
 {
   DLOG(L << "Ueber get() was called for a "<<qtype.getName()<<" record" << endl);
@@ -404,7 +484,7 @@ bool UeberBackend::handle::get(DNSResourceRecord &r)
   while(d_hinterBackend && !(isMore=d_hinterBackend->get(r))) { // this backend out of answers
     if(i<parent->backends.size()) {
       DLOG(L<<"Backend #"<<i<<" of "<<parent->backends.size()
-	   <<" out of answers, taking next"<<endl);
+           <<" out of answers, taking next"<<endl);
       
       d_hinterBackend=parent->backends[i++];
       d_hinterBackend->lookup(qtype,qname,pkt_p);
